package com.seitenbau.testing.addons.dbunit;

import org.dbunit.AbstractDatabaseTester;
import org.dbunit.database.IDatabaseConnection;

/**
 * Interface fr Connection Provider.
 *
 * <p>Ein Connection Provider stellt die aktuelle Connection zur Datenbank bereit.
 * Gerade whrend der Testaufzeichnung ist dies wichtig, da nur so DBUnit auch
 * die 'echten' Daten abgreifen kann, welche innerhalb einer Transaktion vorhanden
 * sind, aber noch nicht commited wurden.
 */
public interface IConnectionProvider
{
  /**
   * _
   *
   * @return
   *    Liefert die DBUnit Database Connection. Die Methode wird vom Aufzeichungscode
   *    gerufen, und sollte/muss die aktuelle Verbindung liefern, so dass Daten direkt
   *    aus der Transaktion abgegriffen werden knnen.
   */
  IDatabaseConnection getConnection();

  /**
   * _
   *
   * @return
   *    Liefert den DBUnit DatabaseTester, der whrend der Testausfhrung ntig ist um
   *    um die Datenbank aufzusetzen bzw. zu Vergleichen.
   */
  AbstractDatabaseTester getDatabaseTester();

  /**
   * Setzt die zum Provider zugehrigen Konfiguration.
   *
   * XXX: Evtl. in konkrete implementierung schieben, da so auch der Type konkreter wird.
   *
   * @param connectionProviderConfig
   *    Die konkrete Konfigurationsinstanz
   */
  void setConfiguration(IConnectionProviderConfig connectionProviderConfig);
}
