package com.seitenbau.testing.addons.dbunit.generator;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.velocity.context.Context;

import com.seitenbau.testing.addons.dbunit.DBUnitGeneratorConstants;
import com.seitenbau.testing.addons.dbunit.listener.DbUnitListenerConfiguration;
import com.seitenbau.testing.addons.dbunit.listener.DbUnitTableConfig;
import com.seitenbau.testing.addons.plugin.DbUnitXStreamSweetner;
import com.seitenbau.testing.shared.helper.FileHelper;
import com.seitenbau.testing.shared.locator.ITestPathFileBasedLocator;
import com.seitenbau.testing.shared.model.ListenerData;
import com.seitenbau.testing.shared.model.RecordedMethodConfiguration;
import com.seitenbau.testing.shared.model.RecordedTest;
import com.seitenbau.testing.shared.model.RecordedTestCase;
import com.seitenbau.testing.testgenerator.partgenerator.velocity.AbstractTemplatePartGenerator;
import com.thoughtworks.xstream.XStream;

/**
 * Part Generator um DBUnit Tests zu generieren.
 */
public class PartGeneratorDBUnit extends AbstractTemplatePartGenerator
{

  private List<RecordedMethodConfiguration> myRecordedMethodList;

  private List<DbUnitTableConfig> tables;

  public void init(List<RecordedMethodConfiguration> config,
      ITestPathFileBasedLocator pathLocator)
  {
    super.init(config, pathLocator);
    createArtifact(pathLocator, config);
  }

  @Override
  protected void prepareContext(Context context)
  {
    super.prepareContext(context);
    context.put("methodlist", getRecordedMethodList());
  }

  public List<String> getImports()
  {
    return Arrays.asList(new String[] {"org.dbunit.JdbcDatabaseTester",
        "org.dbunit.dataset.IDataSet", "org.dbunit.dataset.xml.FlatXmlDataSet",
        "org.dbunit.Assertion", "org.dbunit.dataset.ITable",
        "org.dbunit.dataset.filter.DefaultColumnFilter",
        "com.seitenbau.testing.addons.spring.SpringTestContext",
        "java.io.InputStream", "org.dbunit.JdbcDatabaseTester",
        "org.dbunit.AbstractDatabaseTester",
        "com.seitenbau.testing.addons.dbunit.DBUnitTestHelper"});
  }

  public List<RecordedMethodConfiguration> getRecordedMethodList()
  {
    return myRecordedMethodList;
  }

  protected void createArtifact(ITestPathFileBasedLocator pathLocator,
      List<RecordedMethodConfiguration> cfg)
  {
    for (RecordedMethodConfiguration mcfg : cfg)
    {
      for (RecordedTestCase tCase : mcfg.getRecordedTestCases())
      {
        for (RecordedTest test : tCase.getRecordedTests())
        {
          String filePreState = pathLocator.getTestResourceFile(
              DBUnitGeneratorConstants.PLUGIN_ID, test.getId(),
              DBUnitGeneratorConstants.PRESTATE_FILE_NAME + ".xml");
          String filePostState = pathLocator.getTestResourceFile(
              DBUnitGeneratorConstants.PLUGIN_ID, test.getId(),
              DBUnitGeneratorConstants.POSTSTATE_FILE_NAME + ".xml");

          // String base = pathLocator.getPathToTestResource(
          // pathLocator.getTestResourceRelativeFilePath("dbunit",
          // test.getId(), "") );

          RecordedTestCase tcase = mcfg.getRecordedTestCases().get(0);

          writeToFile(test, DBUnitGeneratorConstants.PLUGIN_ID
              + ".listener:preState", filePreState);
          writeToFile(test, DBUnitGeneratorConstants.PLUGIN_ID
              + ".listener:postState", filePostState);

          ListenerData data = tcase.getData(DBUnitGeneratorConstants.PLUGIN_ID
              + ".listener");
          XStream xs = new XStream();
          new DbUnitXStreamSweetner().registerOnXStream(xs);

          // String configFile=pathLocator.getPathToTestResource(
          // pathLocator.getTestResourceRelativeFilePath("dbunit",
          // "config.xml"));

          String config = data.getBinayData();
          Object obj = xs.fromXML( config );
          if(obj instanceof DbUnitListenerConfiguration)
          {

            String filename = pathLocator.getTestResourceFile(
                DBUnitGeneratorConstants.PLUGIN_ID, "config.xml");

            DbUnitListenerConfiguration oldconfig = (DbUnitListenerConfiguration)obj;

            if(getTables()!=null && !getTables().isEmpty() )
            {
              oldconfig.setTables( getTables() );
            }

            FileHelper.writeFile(filename, xs.toXML(oldconfig));
          }
        }
      }
    }

  }

//  private DbUnitTableConfig getByTableName(List<DbUnitTableConfig> tables2,
//      DbUnitTableConfig second)
//  {
//    for(DbUnitTableConfig tab:tables2)
//    {
//      if(tab.getTableNname().equals(second.getTableNname())) {
//        return tab;
//      }
//    }
//    return null;
//  }

  private void writeToFile(RecordedTest test, String key, String fileName)
  {
    File preStateFile = new File(fileName);
    try
    {
      preStateFile.createNewFile();
      FileOutputStream fo = new FileOutputStream(preStateFile);
      ListenerData data = test.getData(key);
      if (data != null)
      {
        String x = data.getBinayData();
        byte[] b = x.getBytes();
        fo.write(b);
        fo.close();
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  public List<DbUnitTableConfig> getTables()
  {
    if(tables==null)
    {
      tables=new ArrayList<DbUnitTableConfig>();
    }
    return tables;
  }

  public void addTable(DbUnitTableConfig table)
  {
    getTables().add(table);
  }

  public void setTables(List<DbUnitTableConfig> tables)
  {
    this.tables = tables;
  }

}
