package com.seitenbau.testing.addons.dbunit.listener;

import java.io.IOException;
import java.io.StringWriter;
import java.sql.SQLException;

import org.aspectj.lang.JoinPoint;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.database.QueryDataSet;
import org.dbunit.dataset.DataSetException;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.xml.XmlDataSet;

import com.seitenbau.testing.addons.dbunit.DBUnitGeneratorConstants;
import com.seitenbau.testing.addons.dbunit.IConnectionProvider;
import com.seitenbau.testing.addons.plugin.DbUnitXStreamSweetner;
import com.seitenbau.testing.shared.config.IListenerConfiguration;
import com.seitenbau.testing.shared.helper.ReflectionHelper;
import com.seitenbau.testing.shared.model.ListenerData;
import com.seitenbau.testing.shared.model.RecordedTest;
import com.seitenbau.testing.shared.model.RecordedTestCase;
import com.seitenbau.testing.shared.tracer.IAspectJAware;
import com.seitenbau.testing.shared.tracer.listener.DefaultListener;
import com.thoughtworks.xstream.XStream;

/**
 * Listener zur Aufzeichnung von Datenbanken
 */
public class DBUnitListener extends DefaultListener implements IAspectJAware
{

  private DbUnitListenerConfiguration dconf;

  private IConnectionProvider myProvider;

  protected void setup(IListenerConfiguration config)
  {
    if (config instanceof DbUnitListenerConfiguration)
    {
      dconf = (DbUnitListenerConfiguration) config;
      if (dconf.getConnectionProvider() != null)
      {
        IConnectionProvider provider = ReflectionHelper
            .createInstanceSilent(dconf.getConnectionProvider()
                .getConnectionProviderClass());
        if (provider != null)
        {
          myProvider = provider;
          myProvider.setConfiguration(dconf.getConnectionProvider());
        }
      }
    }
    else
    {
      System.err.println("DBUnit Listener , illegal configuration File");
    }
  }

  @Override
  protected void registerOnTestCase(RecordedTestCase theCase)
  {
    theCase.addRecordedFeatures(DBUnitGeneratorConstants.PLUGIN_ID + ".listener");
    XStream xs = new XStream();
    new DbUnitXStreamSweetner().registerOnXStream(xs);
    String xml = xs.toXML(dconf);
    theCase.setData(DBUnitGeneratorConstants.PLUGIN_ID + ".listener",
        new ListenerData(xml));
  }

  @Override
  public void enteringTargetMethod(RecordedTest recordedTest)
  {
    recordedTest.setData(DBUnitGeneratorConstants.PLUGIN_ID + ".listener:preState",
        getDatabaseSnapshot());
  }

  @Override
  public void finished(RecordedTest recordedTest)
  {
    recordedTest.setData(
        DBUnitGeneratorConstants.PLUGIN_ID + ".listener:postState",
        getDatabaseSnapshot());

    // Storing data:
    super.finished(recordedTest);
  }

  private ListenerData getDatabaseSnapshot()
  {
    try
    {
      // Class.forName(dbClass);

      IDatabaseConnection connection = myProvider.getConnection();

      // ByteArrayOutputStream bo=new ByteArrayOutputStream();
      StringWriter sw = new StringWriter();

      if (dconf.getTables().size() > 0)
      {
        // Teil export
        QueryDataSet partialDataSet = new QueryDataSet(connection);

        for (DbUnitTableConfig table : dconf.getTables())
        {
          partialDataSet.addTable(table.getTableNname());
        }

        XmlDataSet.write(partialDataSet, sw);
        // FlatXmlDataSet.write(partialDataSet, sw);
      }
      else
      {
        // Komplett export
        IDataSet fullDataSet = connection.createDataSet();
        XmlDataSet.write(fullDataSet, sw);
        // FlatXmlDataSet.write(fullDataSet, sw);
      }

      connection.close();

      ListenerData ld = new ListenerData(sw.toString());

      return ld;

    }
    catch (SQLException e)
    {
      e.printStackTrace();
    }
    catch (DataSetException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return null;
  }

  public IConnectionProvider getProvider()
  {
    return myProvider;
  }

  public void setProvider(IConnectionProvider provider)
  {
    myProvider = provider;
  }

  public void enteringTarget(JoinPoint thisJoinPoint)
  {
    if (getProvider() instanceof IAspectJAware)
    {
      ((IAspectJAware) getProvider()).enteringTarget(thisJoinPoint);
    }
  }

  public void exceptionThrownTarget(JoinPoint thisJoinPoint,
      Throwable theThrowable)
  {
    if (getProvider() instanceof IAspectJAware)
    {
      ((IAspectJAware) getProvider()).exceptionThrownTarget(thisJoinPoint,
          theThrowable);
    }
  }

  public void proceededTarget(JoinPoint thisJoinPoint)
  {
    if (getProvider() instanceof IAspectJAware)
    {
      ((IAspectJAware) getProvider()).proceededTarget(thisJoinPoint);
    }
  }

}
