package com.seitenbau.testing.addons.dbunit.listener;

import java.util.ArrayList;
import java.util.List;

import com.seitenbau.testing.addons.dbunit.IConnectionProviderConfig;
import com.seitenbau.testing.instrumentation.generator.impl.IPrintHelpMessage;
import com.seitenbau.testing.shared.config.IListenerConfiguration;
import com.seitenbau.testing.shared.tracer.ITracerListener;
import com.seitenbau.testing.shared.writer.IWriter;

/**
 * Konfigurations Optionen fr den DBUnit Listener.
 *
 * <p>
 * Neben der Tabellen Konfiguration ist ein Connection Provider anzugeben, ber
 * welchen die Konkrete Instanz der Datenbank Connection gefunden wird.
 *
 */
public class DbUnitListenerConfiguration implements IListenerConfiguration,
		IPrintHelpMessage {

	private List<DbUnitTableConfig> tables;

	private IConnectionProviderConfig ConnectionProvider;

	public List<DbUnitTableConfig> getTables() {
		if (tables == null) {
			tables = new ArrayList<DbUnitTableConfig>();
		}
		return tables;
	}

	public void setTables(List<DbUnitTableConfig> tables) {
		this.tables = tables;
	}

	public boolean add(DbUnitTableConfig o) {
		return getTables().add(o);
	}

	public boolean remove(DbUnitTableConfig o) {
		return getTables().remove(o);
	}

	public Class<? extends ITracerListener> getListenerClass() {
		return DBUnitListener.class;
	}

	public void setConnectionProvider(IConnectionProviderConfig provider) {
		ConnectionProvider = provider;
	}

	public IConnectionProviderConfig getConnectionProvider() {
		return ConnectionProvider;
	}
	public void printHelpMessage(IWriter writer) {

		writer.appendNewLine("** DBUnit Listener **");
		if( getTables().isEmpty() )
		{
		  writer.appendNewLine(" * No tables specified. Complete Database snapshots will be saved.");
		}
		writer.appendNewLine();

		IConnectionProviderConfig prov = getConnectionProvider();
		if(prov instanceof IPrintHelpMessage) {
			((IPrintHelpMessage)prov).printHelpMessage( writer );
		}
	}

}
