package com.seitenbau.testing.addons.spring.dbunit;

import java.sql.Connection;

import org.dbunit.AbstractDatabaseTester;
import org.dbunit.DefaultDatabaseTester;
import org.dbunit.database.DatabaseConnection;
import org.dbunit.database.IDatabaseConnection;
import org.hibernate.SessionFactory;

import com.seitenbau.testing.addons.dbunit.IConnectionProvider;
import com.seitenbau.testing.addons.dbunit.IConnectionProviderConfig;

/**
 * Provider fr die Hibernate Session in einer Spring2 Umgebung.
 *
 * <p>Hierzu muss auf der Spring Seite die folgende Bean aktiviert sein:
 * <code><pre>
 *  &lt;bean id="SpringConnectionHibernateSessionInjectorBean"
 *   class="com.seitenbau.testing.addons.dbunit.connection.SpringConnectionHibernateSessionInjectorBean"
 *   lazy-init="false"&lt;
 *   &lt;property name="sessionFactory" ref="Ersetzen durch meine Session Factory Bean ID" /&lt;
 * &lt;/bean&lt;
 * </pre></code>
 *
 */
public class ConnectionProviderSpringHibernateSessionProvider
    implements IConnectionProvider
{

  public IDatabaseConnection getConnection()
  {
    SessionFactory factory = SpringConnectionHibernateSessionInjectorBean
        .getStaticSessionFactory();
    if (factory != null)
    {
      org.hibernate.classic.Session session = factory.getCurrentSession();
      if (session != null)
      {
        // Synch objecst and transaction
        session.flush();
        Connection conn = session.connection();
        if (conn != null)
        {
          return new DatabaseConnection(conn);
        }
      }
    }
    // XXX: log error to log
    return null;
  }

  public AbstractDatabaseTester getDatabaseTester()
  {
    return new DefaultDatabaseTester( getConnection() ){
      @Override
      public void closeConnection(IDatabaseConnection connection)
          throws Exception
      {
        // TODO Auto-generated method stub
      }
    };
  }

  public void setConfiguration(
      IConnectionProviderConfig connectionProviderConfig)
  {
  }

}
